/**
  *	Copyright (C) @YEAR@ Vernon Systems Limited
  *
  *	Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
  *	to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
  *	and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
  *
  *	The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
  *
  *	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
  *	WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
package com.ehive.api.domain.objects;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.ehive.api.domain.objects.Attribute.AttributeIdentifier;
import com.ehive.api.exceptions.EHiveApiException;

public class Media implements Serializable {

	private static final long serialVersionUID = 8921192571733590668L;
	
	private String identifier;
	private List<Attribute> attributes = new ArrayList<Attribute>();
	
	
	public Media(String identifier, String attributeValue) throws EHiveApiException {
		if (!MediaIdentifier.isValid(identifier)) throw new EHiveApiException("The Media identifier entered was not a valid eHive Media identifier: \"" + identifier + "\". \nPlease check the API docs online for a list of valid Media identifiers.");
			
		this.identifier = identifier;
		
		Attribute attribute = new Attribute(AttributeIdentifier.VALUE, attributeValue);

		this.getAttributes().add(attribute);
	}
	
	public void addAttribute(String attributeKey, Object attributeValue ) throws EHiveApiException {
		if (!AttributeIdentifier.isValid(attributeKey)) throw new EHiveApiException("The Media identifier entered was not a valid eHive Media identifier: \"" + identifier + "\". \nPlease check the API docs online for a list of valid Media identifiers.");
		
		AttributeIdentifier attributeIdentifier = AttributeIdentifier.getIdentifier(attributeKey);
		
		this.addAttribute(attributeIdentifier, attributeValue);
	}

	public void addAttribute(AttributeIdentifier attributeIdentifier, Object attributeValue ) throws EHiveApiException {	
		if (!attributeIdentifier.isInstance(attributeValue)) throw new EHiveApiException("This Attribute value is not a valid type for this Attribute type: \"" + identifier + "\". \nPlease check the API docs online for a list of valid Media Attributes.");
		
		MediaIdentifier mediaIdentifier = MediaIdentifier.getMediaIdentifier(this.getIdentifier());
		
		Boolean isValid;
		isValid = false;
		
		for (Enum<?> validAttributeIdentifier : mediaIdentifier.getValidAttributeIdentifiers()) {
			if(attributeIdentifier == ((MediaAttributeMapper) validAttributeIdentifier).getAttributeIdentifier()) {
				isValid = true;
				break;
			}
		}

		if (!isValid) {
			throw new EHiveApiException("The Attribute, \"" + attributeIdentifier.toString() + "\" is not valid for the Media type \"" + mediaIdentifier.toString() + "\". \nPlease check the API docs online for a list of valid Media Attributes.");
		} 
		
		if (attributeIdentifier == AttributeIdentifier.VALUE) {
			Integer mediaLength = MediaIdentifier.getMediaIdentifier(this.getIdentifier()).getMediaLength();
			
			if (!(attributeValue.toString().length() <= mediaLength)) throw new EHiveApiException("The value entered for the \"" + mediaIdentifier.toString() + "\" media exceeds the allowed data size. \nPlease refer to the API docs for a list of Media and their data limits.");
		}
		
		Attribute attribute = new Attribute( attributeIdentifier, attributeValue );
		this.getAttributes().add( attribute );
	}
	
	
	
	public String getIdentifier() { return identifier; }
	public void setIdentifier(String identifier) { this.identifier = identifier; }
	
	public List<Attribute> getAttributes() { return attributes; }
	public void setAttributes(List<Attribute> attributes) { this.attributes = attributes; }
	
	interface MediaAttributeMapper {
		public AttributeIdentifier getAttributeIdentifier();
	}
	
	public enum MediaIdentifier {
		IMAGE_O("image_o", ImageOriginalAttributeIdentifier.values(), MediaDataType.IMAGE, 0),
		IMAGE_L("image_l", ImageLargeAttributeIdentifier.values(), MediaDataType.IMAGE, 0),
		IMAGE_M("image_m", ImageMediumAttributeIdentifier.values(), MediaDataType.IMAGE, 0),
		IMAGE_S("image_s", ImageSmallAttributeIdentifier.values(), MediaDataType.IMAGE,0 ),
		IMAGE_T("image_t", ImageTinyAttributeIdentifier.values(), MediaDataType.IMAGE,0 ),
		IMAGE_TS("image_ts", ImageTinySquareAttributeIdentifier.values(), MediaDataType.IMAGE, 0),
		IMAGE_NS("image_ns", ImageNanoSquareAttributeIdentifier.values(), MediaDataType.IMAGE, 0);
		
		public enum MediaDataType {
			TEXT,
			VARCHAR,
			NUMBER,
			IMAGE;
		}
		
		enum ImageOriginalAttributeIdentifier implements MediaAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE),
			HEIGHT(AttributeIdentifier.HEIGHT),
			WIDTH(AttributeIdentifier.WIDTH),
			TITLE(AttributeIdentifier.TITLE),
			URL(AttributeIdentifier.URL),
			BASE64_ENCODED_IMAGE(AttributeIdentifier.BASE64_ENCODED_IMAGE);			
			
			private final AttributeIdentifier attributeIdentifier;
			
			private ImageOriginalAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}
		
		enum ImageLargeAttributeIdentifier implements MediaAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE),
			HEIGHT(AttributeIdentifier.HEIGHT),
			WIDTH(AttributeIdentifier.WIDTH),
			TITLE(AttributeIdentifier.TITLE),
			URL(AttributeIdentifier.URL),
			BASE64_ENCODED_IMAGE(AttributeIdentifier.BASE64_ENCODED_IMAGE);			
			
			private final AttributeIdentifier attributeIdentifier;
			
			private ImageLargeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}
		
		enum ImageMediumAttributeIdentifier implements MediaAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE),
			HEIGHT(AttributeIdentifier.HEIGHT),
			WIDTH(AttributeIdentifier.WIDTH),
			TITLE(AttributeIdentifier.TITLE),
			URL(AttributeIdentifier.URL),
			BASE64_ENCODED_IMAGE(AttributeIdentifier.BASE64_ENCODED_IMAGE);			
			
			private final AttributeIdentifier attributeIdentifier;
			
			private ImageMediumAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}
		
		enum ImageSmallAttributeIdentifier implements MediaAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE),
			HEIGHT(AttributeIdentifier.HEIGHT),
			WIDTH(AttributeIdentifier.WIDTH),
			TITLE(AttributeIdentifier.TITLE),
			URL(AttributeIdentifier.URL),
			BASE64_ENCODED_IMAGE(AttributeIdentifier.BASE64_ENCODED_IMAGE);			
			
			private final AttributeIdentifier attributeIdentifier;
			
			private ImageSmallAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}
		
		enum ImageTinyAttributeIdentifier implements MediaAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE),
			HEIGHT(AttributeIdentifier.HEIGHT),
			WIDTH(AttributeIdentifier.WIDTH),
			TITLE(AttributeIdentifier.TITLE),
			URL(AttributeIdentifier.URL),
			BASE64_ENCODED_IMAGE(AttributeIdentifier.BASE64_ENCODED_IMAGE);			
			
			private final AttributeIdentifier attributeIdentifier;
			
			private ImageTinyAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}
		
		enum ImageTinySquareAttributeIdentifier implements MediaAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE),
			HEIGHT(AttributeIdentifier.HEIGHT),
			WIDTH(AttributeIdentifier.WIDTH),
			TITLE(AttributeIdentifier.TITLE),
			URL(AttributeIdentifier.URL),
			BASE64_ENCODED_IMAGE(AttributeIdentifier.BASE64_ENCODED_IMAGE);			
			
			private final AttributeIdentifier attributeIdentifier;
			
			private ImageTinySquareAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}
		
		enum ImageNanoSquareAttributeIdentifier implements MediaAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE),
			HEIGHT(AttributeIdentifier.HEIGHT),
			WIDTH(AttributeIdentifier.WIDTH),
			TITLE(AttributeIdentifier.TITLE),
			URL(AttributeIdentifier.URL),
			BASE64_ENCODED_IMAGE(AttributeIdentifier.BASE64_ENCODED_IMAGE);			
			
			private final AttributeIdentifier attributeIdentifier;
			
			private ImageNanoSquareAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}
		
		private final String identifier;
		private final Enum<?>[] validAttributeIdentifiers;
		private final MediaDataType mediaDataType;
		private final Integer mediaLength;
		
		private MediaIdentifier(String identifier, Enum<?>[] validAttributeIdentifers, MediaDataType mediaDataType, Integer mediaLength) {
			this.identifier = identifier;
			this.validAttributeIdentifiers = validAttributeIdentifers;
			this.mediaDataType = mediaDataType;
			this.mediaLength = mediaLength;
		}
		
		public String getMediaIdentifier() {
			return this.identifier;
		}
		
		public Enum<?>[] getValidAttributeIdentifiers() {
			return this.validAttributeIdentifiers;
		}
		
		public MediaDataType getMediaDataType() {
			return this.mediaDataType;
		}
		
		public Integer getMediaLength() {
			return this.mediaLength;
		}
		
		public static Boolean isValid(String identifier) {
			try { 
				getMediaIdentifier(identifier); 
			} catch (IllegalArgumentException e) { 
				return false;
			}
			return true;
		}
		
		public static MediaIdentifier getMediaIdentifier(String identifier) {
			return valueOf(identifier.toUpperCase());
		}

		public String toString() {
			return this.identifier;
		}
	}
}