/**
  *	Copyright (C) @YEAR@ Vernon Systems Limited
  *
  *	Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
  *	to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
  *	and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
  *
  *	The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
  *
  *	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
  *	WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
package com.ehive.api.domain.objects;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.ehive.api.domain.objects.Field.FieldIdentifier;
import com.ehive.api.domain.objects.FieldSet.FieldSetFieldMapper;
import com.ehive.api.domain.objects.FieldSet.FieldSetIdentifier;
import com.ehive.api.domain.objects.Media.MediaIdentifier;
import com.ehive.api.domain.objects.MediaSet.MediaSetIdentifier;
import com.ehive.api.domain.objects.MediaSet.MediaSetMediaMapper;
import com.ehive.api.exceptions.EHiveApiException;

public class ObjectRecord implements Serializable {

	private static final long serialVersionUID = 6224553305064897082L;

	private Long objectRecordId;
	private String externalId;	
	private String objectUrl;
	private String slug;
	private String catalogueType;
	private String metadataRights;
	private Integer accountId;
	private Float searchScore;	
	
	private List<FieldSet> fieldSets = new ArrayList<FieldSet>();
	private List<MediaSet> mediaSets = new ArrayList<MediaSet>();
		
	private transient Map<String, Integer> fieldSetIndex = new HashMap<String, Integer>();
	private transient Map<String, Integer> mediaSetIndex = new HashMap<String, Integer>();

	
	public Long getObjectRecordId() { return objectRecordId; }
	public void setObjectRecordId(Long objectRecordId) { this.objectRecordId = objectRecordId; }
	
	public String getExternalId() { return externalId; }
	public void setExternalId(String externalId) { this.externalId = externalId; }
	
	public Integer getAccountId() { return accountId; }
	public void setAccountId(Integer accountId) { this.accountId = accountId; }

	public Float getSearchScore() { return searchScore; }
	public void setSearchScore(Float searchScore) { this.searchScore = searchScore; }

	public List<FieldSet> getFieldSets() { return fieldSets; }
	public void setFieldSets(List<FieldSet> fieldSets) { this.fieldSets = fieldSets; }

	public String getObjectUrl() { return objectUrl; }
	public void setObjectUrl(String objectUrl) { this.objectUrl = objectUrl; }

	public String getSlug() { return slug; }
	public void setSlug(String slug) { this.slug = slug; }

	public String getCatalogueType() { return catalogueType; }
	public void setCatalogueType(String catalogueType) { this.catalogueType = catalogueType; }
	
	public String getMetadataRights() { return metadataRights; }
	public void setMetadataRights(String metadataRights) { this.metadataRights = metadataRights; }	
	
	public Map<String, Integer> getFieldSetIndex() { return fieldSetIndex; }
	public void setFieldSetIndex(Map<String, Integer> fieldSetIndex) { this.fieldSetIndex = fieldSetIndex; }

	public List<MediaSet> getMediaSets() { return mediaSets; }
	public void setMediaSets(List<MediaSet> mediaSets) { this.mediaSets = mediaSets; }	
	

	public void addMediaRowToMediaSet(String identifier, MediaRow mediaRow) throws EHiveApiException {
		if (!MediaSetIdentifier.isValid(identifier)){
			throw new EHiveApiException("The Media Set identifier entered was not a valid eHive Media Set identifier: \"" + identifier + "\". \nPlease check the API docs online for a list of valid Media Set identifiers.");
		}
		
		MediaSetIdentifier mediaSetIdentifier = MediaSetIdentifier.getMediaSetIdentifier(identifier);
		
		this.addMediaRowToMediaSet(mediaSetIdentifier, mediaRow);
	}
	
	public void addMediaRowToMediaSet(MediaSetIdentifier mediaSetIdentifier, MediaRow mediaRow) throws EHiveApiException {

		Boolean isValid;
		for (Media media : mediaRow.getMedia()) {
			isValid = false;
			if (!MediaIdentifier.isValid(media.getIdentifier())) throw new EHiveApiException("The Media identifier entered was not a valid eHive Media identifier: \"" + media.getIdentifier() + "\". \nPlease check the API docs online for a list of valid Media identifiers.");
			
			MediaIdentifier mediaIdentifier = MediaIdentifier.getMediaIdentifier(media.getIdentifier());
			
			for (Enum<?> validMediaIdentifier : mediaSetIdentifier.getValidMediaIdentifiers()) {
				if(mediaIdentifier == ((MediaSetMediaMapper) validMediaIdentifier).getMediaIdentifier()) {
					isValid = true;
					break;
				}
			}

			if (!isValid) {
				throw new EHiveApiException("The Field, \"" + media.getIdentifier() + "\" is not valid for this Field Set. \nPlease check the API docs online for the correct Field groupings.");
			} 
		}

		MediaSet newMediaSet = null;
		
		for (MediaSet mediaSet: this.getMediaSets()) {
			if (mediaSet.getIdentifier() == null) {
				throw new EHiveApiException("A field set Exists that has a NULL identifier.");
			}
			
			if (mediaSet.getIdentifier().equals(mediaSetIdentifier.toString())) {
				newMediaSet = mediaSet;
				break;
			}			
		}
		
		if (newMediaSet == null) {
			newMediaSet = new MediaSet( mediaSetIdentifier.toString() );
			mediaSets.add( newMediaSet );
		}
		
		newMediaSet.getMediaRows().add(mediaRow);
		
		this.mediaSetIndex.clear();
		int position = 0;
		
		for (MediaSet mediaSet : this.mediaSets) {
			mediaSetIndex.put(mediaSet.getIdentifier(), position++);			
		}
	}
	
	
	
	
	public void addFieldRowToFieldSet(String identifier, FieldRow fieldRow) throws EHiveApiException {
		if (!FieldSetIdentifier.isValid(identifier)){
			throw new EHiveApiException("The Field Set identifier entered was not a valid eHive Field Set identifier: \"" + identifier + "\". \nPlease check the API docs online for a list of valid Field Set identifiers.");
		}
		
		FieldSetIdentifier fieldSetIdentifier = FieldSetIdentifier.getFieldSetIdentifier(identifier);
		
		this.addFieldRowToFieldSet(fieldSetIdentifier, fieldRow);
	}
	
	public void addFieldRowToFieldSet(FieldSetIdentifier fieldSetIdentifier, FieldRow fieldRow) throws EHiveApiException {

		// Check if the fields in the field row are valid for this field set
		Boolean isValid;
		for (Field field : fieldRow.getFields()) {
			isValid = false;
			if (!FieldIdentifier.isValid(field.getIdentifier())) throw new EHiveApiException("The Field identifier entered was not a valid eHive Field identifier: \"" + field.getIdentifier() + "\". \nPlease check the API docs online for a list of valid Field identifiers.");
			
			FieldIdentifier fieldIdentifier = FieldIdentifier.getFieldIdentifier(field.getIdentifier());
			
			for (Enum<?> validFieldIdentifier : fieldSetIdentifier.getValidFieldIdentifiers()) {
				if(fieldIdentifier == ((FieldSetFieldMapper) validFieldIdentifier).getFieldIdentifier()) {
					isValid = true;
					break;
				}
			}

			if (!isValid) {
				throw new EHiveApiException("The Field, \"" + field.getIdentifier() + "\" is not valid for this Field Set. \nPlease check the API docs online for the correct Field groupings.");
			} 
		}

		FieldSet newFieldSet = null;
		
		for (FieldSet fieldSet: this.getFieldSets()) {
			if (fieldSet.getIdentifier() == null) {
				throw new EHiveApiException("A field set Exists that has a NULL identifier.");
			}
			
			if (fieldSet.getIdentifier().equals(fieldSetIdentifier.toString())) {
				newFieldSet = fieldSet;
				break;
			}			
		}
		
		if (newFieldSet == null) {
			newFieldSet = new FieldSet( fieldSetIdentifier.toString() );
			fieldSets.add( newFieldSet );
		}
		
		newFieldSet.getFieldRows().add(fieldRow);
		
		this.fieldSetIndex.clear();
		int position = 0;
		
		for (FieldSet fieldSet : this.fieldSets) {
			fieldSetIndex.put(fieldSet.getIdentifier(), position++);			
		}
	}
	
	public FieldSet getFieldSetByIdentifier(String fieldIdentifier) {
		if (this.fieldSetIndex.containsKey(fieldIdentifier)) {
			int index = this.fieldSetIndex.get(fieldIdentifier);
			FieldSet fieldSet = this.getFieldSets().get(index);
			
			return fieldSet;
		}
		return null;
	}
	
	public MediaSet getMediaSetByIdentifier(String fieldIdentifier) {
		if (this.mediaSetIndex.containsKey(fieldIdentifier)) {
			int index = this.mediaSetIndex.get(fieldIdentifier);
			MediaSet mediaSet = this.getMediaSets().get(index);
			
			return mediaSet;
		}
		return null;
	}
	
	public FieldSet getFieldSetByIdentifier(FieldSetIdentifier fieldSetIdentifier) {
		return this.getFieldSetByIdentifier(fieldSetIdentifier.toString());
	}
}