/**
  *	Copyright (C) @YEAR@ Vernon Systems Limited
  *
  *	Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
  *	to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
  *	and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
  *
  *	The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
  *
  *	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
  *	WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
package com.ehive.api.domain.objects;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import com.ehive.api.domain.objects.Attribute.AttributeIdentifier;
import com.ehive.api.exceptions.EHiveApiException;

public class Field implements Serializable {

	private static final long serialVersionUID = -1593536552459180342L;
	
	private String identifier;
	private List<Attribute> attributes = new ArrayList<Attribute>();
		
	public Field() {}
		
	public Field(String identifier, String attributeValue) throws EHiveApiException {
		if (!FieldIdentifier.isValid(identifier)) throw new EHiveApiException("The Field identifier entered was not a valid eHive Field identifier: \"" + identifier + "\". \nPlease check the API docs online for a list of valid Field identifiers.");
			
		this.identifier = identifier;
		
		Attribute attribute = new Attribute(AttributeIdentifier.VALUE, attributeValue);

		this.getAttributes().add(attribute);
	}

	public Field(FieldIdentifier fieldIdentifier, String attributeValue) throws EHiveApiException {
		this.identifier = fieldIdentifier.toString();
		
		Attribute attribute = new Attribute(AttributeIdentifier.VALUE, attributeValue);

		this.getAttributes().add(attribute);
	}
	
	public void addAttribute(String attributeKey, Object attributeValue ) throws EHiveApiException {
		if (!AttributeIdentifier.isValid(attributeKey)) throw new EHiveApiException("The Field identifier entered was not a valid eHive Field identifier: \"" + identifier + "\". \nPlease check the API docs online for a list of valid Field identifiers.");
		
		AttributeIdentifier attributeIdentifier = AttributeIdentifier.getIdentifier(attributeKey);
		
		this.addAttribute(attributeIdentifier, attributeValue);
	}

	public void addAttribute(AttributeIdentifier attributeIdentifier, Object attributeValue ) throws EHiveApiException {	
		
		if (!attributeIdentifier.isInstance(attributeValue)) throw new EHiveApiException("This Attribute value is not a valid type for this Attribute type: \"" + identifier + "\". \nPlease check the API docs online for a list of valid Field Attributes.");
		
		FieldIdentifier fieldIdentifier = FieldIdentifier.getFieldIdentifier(this.getIdentifier());
		
		Boolean isValid;
		isValid = false;
		
		
		for (Enum<?> validAttributeIdentifier : fieldIdentifier.getValidAttributeIdentifiers()) {
			if(attributeIdentifier == ((FieldAttributeMapper) validAttributeIdentifier).getAttributeIdentifier()) {
				isValid = true;
				break;
			}
		}

		if (!isValid) {
			throw new EHiveApiException("The Attribute, \"" + attributeIdentifier.toString() + "\" is not valid for the Field type \"" + fieldIdentifier.toString() + "\". \nPlease check the API docs online for a list of valid Field Attributes.");
		} 
		
		if (attributeIdentifier == AttributeIdentifier.VALUE) {
			Integer fieldLength = FieldIdentifier.getFieldIdentifier(this.getIdentifier()).getFieldLength();
			
			switch (fieldIdentifier.getFieldDataType()) {
			case POINTER:
			case RELATIONSHIP:
			case ACCOUNT_RELATIONSHIP:
				break;
			default:
				if (!(attributeValue.toString().length() <= fieldLength)) throw new EHiveApiException("The value entered for the \"" + fieldIdentifier.toString() + "\" field exceeds the allowed data size. \nPlease refer to the API docs for a list of Fields and their data limits.");
				break;
			}
		}
		
		Attribute attribute = new Attribute( attributeIdentifier, attributeValue );
		this.getAttributes().add( attribute );
	}
	
	public Object getAttributeValueByAttributeKey(String attributeKey) throws EHiveApiException {
		if (!AttributeIdentifier.isValid(attributeKey)) {
			throw new EHiveApiException("The Field identifier entered was not a valid eHive Field identifier: \"" + identifier + "\". \nPlease check the API docs online for a list of valid Field identifiers.");
		}
		
		for (Attribute attribute: this.getAttributes()) {
			if ( attribute.getKey().equals(attributeKey) ) {
				Object value = attribute.getValue().toString();
				return value;
			}
		}
		return "";
	}

	
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((attributes == null) ? 0 : attributes.hashCode());
		result = prime * result
				+ ((identifier == null) ? 0 : identifier.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Field other = (Field) obj;
		if (attributes == null) {
			if (other.attributes != null)
				return false;
		} else if (!attributes.equals(other.attributes))
			return false;
		if (identifier == null) {
			if (other.identifier != null)
				return false;
		} else if (!identifier.equals(other.identifier))
			return false;
		return true;
	}
	
	public String getIdentifier() { return identifier; }
	public void setIdentifier(String fieldIdentifier) { this.identifier = fieldIdentifier; }
	
	public List<Attribute> getAttributes() { return attributes; }
	public void setAttributes(List<Attribute> attributes) { this.attributes = attributes; }
	
	public String getLabel() {
		for (Attribute attribute : this.getAttributes()) {
			if (attribute.getKey().equals(AttributeIdentifier.LABEL.toString())) {
				return attribute.getValue().toString();
			}
		}
		return null;
	} 
	
	public List<Object> getValues() {
		List<Object> values = new ArrayList<Object>();
		
		for (Attribute attribute : this.getAttributes()) {
			if (attribute.getKey().equals(AttributeIdentifier.VALUE)) {
				values.add(attribute.getValue());
			}
		}
		return values;
	}
	
	interface FieldAttributeMapper {
		public AttributeIdentifier getAttributeIdentifier();
	}
	
	public enum FieldIdentifier {
		TRANSFERRED_TO ("transferred_to", TransferredToAttributeIdentifier.values(), FieldDataType.TERM, 200),
		ACCESSION_DATE ("accession_date", AccessionDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		SUBJECT_ASSOCIATION_DESCRIPTION ("subject_association_description", SubjectAssociationDescriptionAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		ASSOCIATION_KEYWORD ("association_keyword", AssociationKeywordAttributeIdentifier.values(), FieldDataType.TERM, 200),
		BRIEF_DESCRIPTION ("brief_description", BriefDescriptionAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		CARRIER_TYPE ("carrier_type", CarrierTypeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		CATALOGUED_DATE ("catalogued_date", CataloguedDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		CATALOGUER ("cataloguer", CataloguerAttributeIdentifier.values(), FieldDataType.TERM, 200),
		CLASSIFICATION ("classification", ClassificationAttributeIdentifier.values(), FieldDataType.TERM, 200),
		COMMENTS ("comments", CommentsAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		COMMENTS_DATE ("comments_date", CommentsDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		COMMENTS_PERSON ("comments_person", CommentsPersonAttributeIdentifier.values(), FieldDataType.TERM, 200),
		COMPLETED_TREATMENT_DESCRIPTION ("completed_treatment_description", CompletedTreatmentDescriptionAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		COMPLETED_TREATMENT_DATE ("completed_treatment_date", CompletedTreatmentDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		COMPLETED_TREATMENT_KEYWORD ("completed_treatment_keyword", CompletedTreatmentKeywordAttributeIdentifier.values(), FieldDataType.TERM, 200),
		COMPLETED_TREATMENT_NOTES ("completed_treatment_notes", CompletedTreatmentNotesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		COMPLETED_TREATMENT_PART_REFERENCE ("completed_treatment_part_reference", CompletedTreatmentPartReferenceAttributeIdentifier.values(), FieldDataType.VARCHAR, 0),
		COMPLETED_TREATMENT_PERSON ("completed_treatment_person", CompletedTreatmentPersonAttributeIdentifier.values(), FieldDataType.TERM, 200),
		CONDITION_AT_ACQUISITION ("condition_at_acquisition", ConditionAtAcquisitionAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		CONDITION_DATE ("condition_date", ConditionDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		CONDITION_KEYWORD ("condition_keyword", ConditionKeywordAttributeIdentifier.values(), FieldDataType.TERM, 200),
		CONDITION_NOTES ("condition_notes", ConditionNotesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		CONDITION_PART_REFERENCE_ASPECT ("condition_part_reference_aspect", ConditionPartReferenceAspectAttributeIdentifier.values(), FieldDataType.VARCHAR, 0),
		CONDITION_PERSON ("condition_person", ConditionPersonAttributeIdentifier.values(), FieldDataType.TERM, 200),
		CONSERVATION_PRIORITY ("conservation_priority", ConservationPriorityAttributeIdentifier.values(), FieldDataType.TERM, 200),
		CONTROLLING_INSTITUTION ("controlling_institution", ControllingInstitutionAttributeIdentifier.values(), FieldDataType.TERM, 200),
		COPYRIGHT_EXPIRY_DATE ("copyright_expiry_date", CopyrightExpiryDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		COPYRIGHT_NOTES ("copyright_notes", CopyrightNotesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		COPYRIGHT_OWNER ("copyright_owner", CopyrightOwnerAttributeIdentifier.values(), FieldDataType.TERM, 200),
		COPYRIGHT_START_DATE ("copyright_start_date", CopyrightStartDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		COPYRIGHT_TYPE ("copyright_type", CopyrightTypeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		CREDIT_LINE ("credit_line", CreditLineAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		CURRENT_CONDITION_DESCRIPTION ("current_condition_description", CurrentConditionDescriptionAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		CURRENT_RECORD_STATUS ("current_record_status", CurrentRecordStatusAttributeIdentifier.values(), FieldDataType.TERM, 200),
		DATE_MADE ("date_made", DateMadeAttributeIdentifier.values(), FieldDataType.DATE, 40),
		DEACCESSION_AUTHORISER ("deaccession_authoriser", DeaccessionAuthoriserAttributeIdentifier.values(), FieldDataType.TERM, 200),
		DEACCESSION_AUTHORISER_POSITION ("deaccession_authoriser_position", DeaccessionAuthoriserPositionAttributeIdentifier.values(), FieldDataType.VARCHAR, 200),
		DEACCESSION_DATE ("deaccession_date", DeaccessionDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		DEACCESSION_METHOD ("deaccession_method", DeaccessionMethodAttributeIdentifier.values(), FieldDataType.TERM, 200),
		DEACCESSION_PROCEEDS ("deaccession_proceeds", DeaccessionProceedsAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		DEACCESSION_REASON ("deaccession_reason", DeaccessionReasonAttributeIdentifier.values(), FieldDataType.TERM, 200),
		DEACCESSION_STATUS ("deaccession_status", DeaccessionStatusAttributeIdentifier.values(), FieldDataType.TERM, 200),
		DEPARTMENT ("department", DepartmentAttributeIdentifier.values(), FieldDataType.TERM, 200),
		DISPLAY_DETAILS ("display_details", DisplayDetailsAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		DUBLIN_CORE ("dublin_core", DublinCoreAttributeIdentifier.values(), FieldDataType.GLOBAL_TERM, 200),
		EDITION ("edition", EditionAttributeIdentifier.values(), FieldDataType.VARCHAR, 200),
		FIELD_COLLECTION_DATE ("field_collection_date", FieldCollectionDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		FIELD_COLLECTION_TIME ("field_collection_time", FieldCollectionTimeAttributeIdentifier.values(), FieldDataType.VARCHAR, 50),
		FIELD_COLLECTION_METHOD ("field_collection_method", FieldCollectionMethodAttributeIdentifier.values(), FieldDataType.TERM, 200),
		FIELD_COLLECTION_DESCRIPTION ("field_collection_description", FieldCollectionDescriptionAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		FIELD_COLLECTION_PLACE_KEYWORD ("field_collection_place_keyword", FieldCollectionPlaceKeywordAttributeIdentifier.values(), FieldDataType.TERM, 200),
		FIELD_COLLECTION_REFERENCE ("field_collection_reference", FieldCollectionReferenceAttributeIdentifier.values(), FieldDataType.VARCHAR, 50),
		FIELD_COLLECTOR ("field_collector", FieldCollectorAttributeIdentifier.values(), FieldDataType.TERM, 200),
		FIELD_COLLECTOR_ROLE ("field_collector_role", FieldCollectorRoleAttributeIdentifier.values(), FieldDataType.TERM, 200),
		FILM_TYPE ("film_type", FilmTypeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		FORMAT ("format", FormatAttributeIdentifier.values(), FieldDataType.TERM, 200),
		FUNDER ("funder", FunderAttributeIdentifier.values(), FieldDataType.TERM, 200),
		FUNDER_AMOUNT ("funder_amount", FunderAmountAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		FUNDER_TYPE ("funder_type", FunderTypeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		GENERAL_FLAG_ADMIN ("general_flag_admin", GeneralFlagAdminAttributeIdentifier.values(), FieldDataType.TERM, 200),
		GENERAL_NOTES_ACQUISITION ("general_notes_acquisition", GeneralNotesAcquisitionAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		GENERAL_NOTES_ADMIN ("general_notes_admin", GeneralNotesAdminAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		GENERAL_NOTES_OBJECT ("general_notes_object", GeneralNotesObjectAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		GENERAL_NOTES_PLACE_MARK ("general_notes_place_mark", GeneralNotesPlaceMarkAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		GEOLOGICAL_AGE ("geological_age", GeologicalAgeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		GEOLOGICAL_FORMATION ("geological_formation", GeologicalFormationAttributeIdentifier.values(), FieldDataType.TERM, 200),
		HABITAT_DESCRIPTION ("habitat_description", HabitatDescriptionAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		HABITAT_KEYWORD ("habitat_keyword", HabitatKeywordAttributeIdentifier.values(), FieldDataType.TERM, 200),
		HANDLING_DETAILS ("handling_details", HandlingDetailsAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		INSCRIPTION ("inscription", InscriptionAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		ISBN_ISSN ("isbn_issn", IsbnIssnAttributeIdentifier.values(), FieldDataType.VARCHAR, 50),
		ITEM_COUNT ("item_count", ItemCountAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		ITEM_COUNT_NOTES ("item_count_notes", ItemCountNotesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		LANGUAGE ("language", LanguageAttributeIdentifier.values(), FieldDataType.TERM, 200),
		LOCATION ("location", LocationAttributeIdentifier.values(), FieldDataType.TERM, 200),
		LOCATION_NOTES ("location_notes", LocationNotesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		MARINE_ENVIRONMENT ("marine_environment", MarineEnvironmentAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		MEASUREMENT_ASPECT ("measurement_aspect", MeasurementAspectAttributeIdentifier.values(), FieldDataType.TERM, 200),
		MEASUREMENT_DESCRIPTION ("measurement_description", MeasurementDescriptionAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		MEASUREMENT_PART ("measurement_part", MeasurementPartAttributeIdentifier.values(), FieldDataType.VARCHAR, 0),
		MEASUREMENT_UNIT ("measurement_unit", MeasurementUnitAttributeIdentifier.values(), FieldDataType.GLOBAL_TERM, 200),
		MEASUREMENT_VALUE ("measurement_value", MeasurementValueAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		MEDIUM_DESCRIPTION ("medium_description", MediumDescriptionAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		MEDIUM_MATERIAL ("medium_material", MediumMaterialAttributeIdentifier.values(), FieldDataType.TERM, 200),
		MOUNTING_FRAMING ("mounting_framing", MountingFramingAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		NAMED_COLLECTION ("named_collection", NamedCollectionAttributeIdentifier.values(), FieldDataType.TERM, 200),
		OBJECT_NUMBER ("object_number", ObjectNumberAttributeIdentifier.values(), FieldDataType.VARCHAR, 50),
		OBJECT_TYPE ("object_type", ObjectTypeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		OTHER_MAKER_CONTRIBUTOR ("other_maker_contributor", OtherMakerContributorAttributeIdentifier.values(), FieldDataType.TERM, 200),
		OTHER_MAKER_CONTRIBUTOR_ROLE ("other_maker_contributor_role", OtherMakerContributorRoleAttributeIdentifier.values(), FieldDataType.TERM, 200),
		OTHER_NAME_TITLE ("other_name_title", OtherNameTitleAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		OTHER_NAME_TITLE_NOTES ("other_name_title_notes", OtherNameTitleNotesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		OTHER_NAME_TITLE_TYPE ("other_name_title_type", OtherNameTitleTypeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		OTHER_NUMBER ("other_number", OtherNumberAttributeIdentifier.values(), FieldDataType.VARCHAR, 200),
		OTHER_NUMBER_TYPE ("other_number_type", OtherNumberTypeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		PACKING_DETAILS ("packing_details", PackingDetailsAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		PART_ID ("part_id", PartIdAttributeIdentifier.values(), FieldDataType.VARCHAR, 50),
		PART_DESCRIPTION ("part_description", PartDescriptionAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		PART_HOLDINGS ("part_holdings", PartHoldingsAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		PHYSICAL_CHARACTERISTICS ("physical_characteristics", PhysicalCharacteristicsAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		PLACE_MARK ("place_mark", PlaceMarkAttributeIdentifier.values(), FieldDataType.TERM, 200),
		PLACE_MADE ("place_made", PlaceMadeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		PLACE_NOTES ("place_notes", PlaceNotesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		PORTFOLIO_TITLE ("portfolio_title", PortfolioTitleAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		PRICE_FOREIGN ("price_foreign", PriceForeignAttributeIdentifier.values(), FieldDataType.VARCHAR, 50),
		PRICE_LOCAL ("price_local", PriceLocalAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		PRIMARY_CREATOR_MAKER ("primary_creator_maker", PrimaryCreatorMakerAttributeIdentifier.values(), FieldDataType.TERM, 200),
		PRIMARY_CREATOR_MAKER_ROLE ("primary_creator_maker_role", PrimaryCreatorMakerRoleAttributeIdentifier.values(), FieldDataType.TERM, 200),
		PROPOSED_TREATMENT_DESCRIPTION ("proposed_treatment_description", ProposedTreatmentDescriptionAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		PROPOSED_TREATMENT_DATE ("proposed_treatment_date", ProposedTreatmentDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		PROPOSED_TREATMENT_KEYWORD ("proposed_treatment_keyword", ProposedTreatmentKeywordAttributeIdentifier.values(), FieldDataType.TERM, 200),
		PROPOSED_TREATMENT_NOTES ("proposed_treatment_notes", ProposedTreatmentNotesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		PROPOSED_TREATMENT_PART_REFERENCE ("proposed_treatment_part_reference", ProposedTreatmentPartReferenceAttributeIdentifier.values(), FieldDataType.VARCHAR, 0),
		PROPOSED_TREATMENT_PERSON ("proposed_treatment_person", ProposedTreatmentPersonAttributeIdentifier.values(), FieldDataType.TERM, 200),
		PROVENANCE_DATE ("provenance_date", ProvenanceDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		PROVENANCE_DETAILS ("provenance_details", ProvenanceDetailsAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		PROVENANCE_PERSON ("provenance_person", ProvenancePersonAttributeIdentifier.values(), FieldDataType.TERM, 200),
		PROVENANCE_PLACE ("provenance_place", ProvenancePlaceAttributeIdentifier.values(), FieldDataType.TERM, 200),
		PUBLISHER ("publisher", PublisherAttributeIdentifier.values(), FieldDataType.TERM, 200),
		PUBLICATION_DATE ("publication_date", PublicationDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		PUBLICATION_PLACE ("publication_place", PublicationPlaceAttributeIdentifier.values(), FieldDataType.TERM, 200),
		RECORD_STATUS ("record_status", RecordStatusAttributeIdentifier.values(), FieldDataType.TERM, 200),
		RELATED_ACQUISITION_NUMBER ("related_acquisition_number", RelatedAcquisitionNumberAttributeIdentifier.values(), FieldDataType.VARCHAR, 50),
		RELATED_ACQUISITION_NUMBER_TYPE ("related_acquisition_number_type", RelatedAcquisitionNumberTypeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		RELATED_OBJECT ("related_object", RelatedObjectAttributeIdentifier.values(), FieldDataType.TERM, 200),
		RELATED_OBJECT_NOTES ("related_object_notes", RelatedObjectNotesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		RELATED_OBJECT_TYPE ("related_object_type", RelatedObjectTypeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		RESTRICTION_TYPE ("restriction_type", RestrictionTypeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		RISK_ASSESSED_BY ("risk_assessed_by", RiskAssessedByAttributeIdentifier.values(), FieldDataType.TERM, 200),
		RISK_ASSESSMENT ("risk_assessment", RiskAssessmentAttributeIdentifier.values(), FieldDataType.TERM, 200),
		RISK_ASSESSMENT_DATE ("risk_assessment_date", RiskAssessmentDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		RISK_ASSESSMENT_NOTES ("risk_assessment_notes", RiskAssessmentNotesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		RISK_FACTOR ("risk_factor", RiskFactorAttributeIdentifier.values(), FieldDataType.TERM, 200),
		SERIES_NUMBER ("series_number", SeriesNumberAttributeIdentifier.values(), FieldDataType.VARCHAR, 50),
		SERIES_TITLE ("series_title", SeriesTitleAttributeIdentifier.values(), FieldDataType.TERM, 200),
		SEX ("sex", SexAttributeIdentifier.values(), FieldDataType.TERM, 200),
		SHORE_ENVIRONMENT ("shore_environment", ShoreEnvironmentAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		SIGNIFICANCE ("significance", SignificanceAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		SITE_DESCRIPTION ("site_description", SiteDescriptionAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		SITE_KEYWORD ("site_keyword", SiteKeywordAttributeIdentifier.values(), FieldDataType.TERM, 200),
		SOUND_AUDIO ("sound_audio", SoundAudioAttributeIdentifier.values(), FieldDataType.TERM, 200),
		SPECIMEN_AGE ("specimen_age", SpecimenAgeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		SPECIMEN_CATEGORY ("specimen_category", SpecimenCategoryAttributeIdentifier.values(), FieldDataType.TERM, 200),
		SPECIMEN_CATEGORY_NOTES_COUNT ("specimen_category_notes_count", SpecimenCategoryNotesCountAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		SPECIMEN_DETAILS ("specimen_details", SpecimenDetailsAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		SPINE_LABEL ("spine_label", SpineLabelAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		STORAGE_DETAILS ("storage_details", StorageDetailsAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		STORAGE_PRESERVATION_METHOD ("storage_preservation_method", StoragePreservationMethodAttributeIdentifier.values(), FieldDataType.TERM, 200),
		STRATIGRAPHY_DESCRIPTION ("stratigraphy_description", StratigraphyDescriptionAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		STRATIGRAPHY_KEYWORD ("stratigraphy_keyword", StratigraphyKeywordAttributeIdentifier.values(), FieldDataType.TERM, 200),
		STYLE_ICONOGRAPHY ("style_iconography", StyleIconographyAttributeIdentifier.values(), FieldDataType.TERM, 200),
		SUBJECT_KEYWORD ("subject_keyword", SubjectKeywordAttributeIdentifier.values(), FieldDataType.TERM, 200),
		TAXON_DETERMINED_BY ("taxon_determined_by", TaxonDeterminedByAttributeIdentifier.values(), FieldDataType.TERM, 200),
		TAXON_DETERMINED_DATE ("taxon_determined_date", TaxonDeterminedDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		TAXONOMIC_CLASSIFICATION ("taxonomic_classification", TaxonomicClassificationAttributeIdentifier.values(), FieldDataType.TERM, 200),
		TAXONOMIC_CLASSIFICATION_NOTES ("taxonomic_classification_notes", TaxonomicClassificationNotesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		TAXONOMIC_QUALIFIER ("taxonomic_qualifier", TaxonomicQualifierAttributeIdentifier.values(), FieldDataType.TERM, 200),
		TAXONOMIC_TYPE_INDICATOR ("taxonomic_type_indicator", TaxonomicTypeIndicatorAttributeIdentifier.values(), FieldDataType.TERM, 200),
		TECHNIQUE ("technique", TechniqueAttributeIdentifier.values(), FieldDataType.TERM, 200),
		NAME ("name", NameAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		WEB_PUBLIC_DESCRIPTION ("web_public_description", WebPublicDescriptionAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		EXTERNAL_DATABASE_ID ("external_database_id", ExternalDatabaseIdAttributeIdentifier.values(), FieldDataType.VARCHAR, 200),
		SIMPLE_SUMMARY ("simple_summary", SimpleSummaryAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		SUBJECT_ASSOCIATION_EVENT ("subject_association_event", SubjectAssociationEventAttributeIdentifier.values(), FieldDataType.POINTER, 0),
		TAG ("tag", TagAttributeIdentifier.values(), FieldDataType.VARCHAR, 200),
		FAMILY_NAME ("family_name", FamilyNameAttributeIdentifier.values(), FieldDataType.VARCHAR, 200),
		GIVEN_NAME ("given_name", GivenNameAttributeIdentifier.values(), FieldDataType.VARCHAR, 200),
		ADDITIONAL_NAMES ("additional_names", AdditionalNamesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		KNOWN_AS ("known_as", KnownAsAttributeIdentifier.values(), FieldDataType.VARCHAR, 200),
		TITLE ("title", TitleAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		POSTNOMINALS ("postnominals", PostnominalsAttributeIdentifier.values(), FieldDataType.VARCHAR, 200),
		ORGANISATION_NAME ("organisation_name", OrganisationNameAttributeIdentifier.values(), FieldDataType.VARCHAR, 200),
		SORT_NAME ("sort_name", SortNameAttributeIdentifier.values(), FieldDataType.VARCHAR, 200),
		OTHER_NAME_TYPE ("other_name_type", OtherNameTypeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		OTHER_NAME ("other_name", OtherNameAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		OTHER_NAME_DATE ("other_name_date", OtherNameDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		DATE_OF_BIRTH ("date_of_birth", DateOfBirthAttributeIdentifier.values(), FieldDataType.DATE, 40),
		DATE_OF_DEATH ("date_of_death", DateOfDeathAttributeIdentifier.values(), FieldDataType.DATE, 40),
		PLACE_OF_BIRTH ("place_of_birth", PlaceOfBirthAttributeIdentifier.values(), FieldDataType.TERM, 200),
		PLACE_OF_DEATH ("place_of_death", PlaceOfDeathAttributeIdentifier.values(), FieldDataType.TERM, 200),
		URL ("url", UrlAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		RELATED_RECORD ("related_record", RelatedRecordAttributeIdentifier.values(), FieldDataType.POINTER, 0),
		RELATIONSHIP ("relationship", RelationshipAttributeIdentifier.values(), FieldDataType.TERM, 200),
		RELATED_NOTES ("related_notes", RelatedNotesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		JOB_TITLE ("job_title", JobTitleAttributeIdentifier.values(), FieldDataType.VARCHAR, 200),
		JOB_ROLE ("job_role", JobRoleAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		GENERAL_NOTES ("general_notes", GeneralNotesAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		ACTIVE_DATE ("active_date", ActiveDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		ACTIVE_PLACE ("active_place", ActivePlaceAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		ACTIVE_NOTES ("active_notes", ActiveNotesAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		NATIONALITY ("nationality", NationalityAttributeIdentifier.values(), FieldDataType.VARCHAR, 200),
		NATIONALITY_DATE ("nationality_date", NationalityDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		NATIONALITY_NOTES ("nationality_notes", NationalityNotesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		ETHNICITY ("ethnicity", EthnicityAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		ACQ_NUMBER ("acq_number", AcqNumberAttributeIdentifier.values(), FieldDataType.VARCHAR, 50),
		ACQ_METHOD ("acq_method", AcqMethodAttributeIdentifier.values(), FieldDataType.TERM, 200),
		ACQ_REASON ("acq_reason", AcqReasonAttributeIdentifier.values(), FieldDataType.TERM, 200),
		ACQ_DATE ("acq_date", AcqDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		ACQ_AUTHORITY ("acq_authority", AcqAuthorityAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		ACQ_SOURCE ("acq_source", AcqSourceAttributeIdentifier.values(), FieldDataType.TERM, 200),
		ACQ_SOURCE_ROLE ("acq_source_role", AcqSourceRoleAttributeIdentifier.values(), FieldDataType.TERM, 200),
		LOT_VALUATION ("lot_valuation", LotValuationAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		LOT_PRICE_LOCAL ("lot_price_local", LotPriceLocalAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		LOT_PRICE_FOREIGN ("lot_price_foreign", LotPriceForeignAttributeIdentifier.values(), FieldDataType.VARCHAR, 50),
		LOT_DESCRIPTION ("lot_description", LotDescriptionAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		ACQ_VALUATION ("acq_valuation", AcqValuationAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		LEGAL_STATUS ("legal_status", LegalStatusAttributeIdentifier.values(), FieldDataType.TERM, 200),
		LOAN_IN ("loan_in", LoanInAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		LOAN_OUT ("loan_out", LoanOutAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		EXHIBITION_HISTORY ("exhibition_history", ExhibitionHistoryAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		RESEARCH_ATTRIBUTION ("research_attribution", ResearchAttributionAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		RESEARCH_ATTRIBUTION_DATE ("research_attribution_date", ResearchAttributionDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		RESEARCH_ATTRIBUTION_PERSON ("research_attribution_person", ResearchAttributionPersonAttributeIdentifier.values(), FieldDataType.TERM, 200),
		FIELD_COLLECTION_NOTES ("field_collection_notes", FieldCollectionNotesAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		GEOLOGICAL_FORMATION_DESCRIPTION ("geological_formation_description", GeologicalFormationDescriptionAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		GEOLOGICAL_AGE_DESCRIPTION ("geological_age_description", GeologicalAgeDescriptionAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		GEOLOGICAL_FORM ("geological_form", GeologicalFormAttributeIdentifier.values(), FieldDataType.TERM, 200),
		PERIOD ("period", PeriodAttributeIdentifier.values(), FieldDataType.TERM, 200),
		TYPE_ID_RELIABILITY_LEVEL ("type_id_reliability_level", TypeIdReliabilityLevelAttributeIdentifier.values(), FieldDataType.TERM, 200),
		ACQ_STATUS ("acq_status", AcqStatusAttributeIdentifier.values(), FieldDataType.GLOBAL_TERM, 200),
		ACQ_FUNDER ("acq_funder", AcqFunderAttributeIdentifier.values(), FieldDataType.TERM, 200),
		ACQ_FUNDING_AMOUNT ("acq_funding_amount", AcqFundingAmountAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		ACQ_FUNDING_TYPE ("acq_funding_type", AcqFundingTypeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		PRODUCTION_NOTES ("production_notes", ProductionNotesAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		VALUATION_TYPE ("valuation_type", ValuationTypeAttributeIdentifier.values(), FieldDataType.GLOBAL_TERM, 200),
		VALUATION_METHOD ("valuation_method", ValuationMethodAttributeIdentifier.values(), FieldDataType.TERM, 200),
		VALUATION ("valuation", ValuationAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		VALUATION_VALUER ("valuation_valuer", ValuationValuerAttributeIdentifier.values(), FieldDataType.TERM, 200),
		VALUATION_DATE ("valuation_date", ValuationDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		VALUATION_FOREIGN_CURRENCY_TYPE ("valuation_foreign_currency_type", ValuationForeignCurrencyTypeAttributeIdentifier.values(), FieldDataType.TERM, 200),
		VALUATION_FOREIGN_CURRENCY_VALUE ("valuation_foreign_currency_value", ValuationForeignCurrencyValueAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		VALUATION_EXCHANGE_RATE ("valuation_exchange_rate", ValuationExchangeRateAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		VALUATION_NOTES ("valuation_notes", ValuationNotesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		LATEST_VALUATION ("latest_valuation", LatestValuationAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		LATEST_APPRAISAL_VALUATION ("latest_appraisal_valuation", LatestAppraisalValuationAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		LATEST_CONSERVATION_VALUATION ("latest_conservation_valuation", LatestConservationValuationAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		LATEST_INSURANCE_VALUATION ("latest_insurance_valuation", LatestInsuranceValuationAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		LATEST_OUTGOING_LOAN_VALUATION ("latest_outgoing_loan_valuation", LatestOutgoingLoanValuationAttributeIdentifier.values(), FieldDataType.NUMBER, 25),
		LOCATION_HISTORY ("location_history", LocationHistoryAttributeIdentifier.values(), FieldDataType.TERM, 200),
		LOCATION_HISTORY_DATE ("location_history_date", LocationHistoryDateAttributeIdentifier.values(), FieldDataType.DATE, 40),
		LOCATION_HISTORY_NOTES ("location_history_notes", LocationHistoryNotesAttributeIdentifier.values(), FieldDataType.VARCHAR, 1000),
		RECORD_ACQUISITION_LOT_RELATIONSHIP ("record_acquisition_lot_relationship", RecordAcquisitionLotRelationshipAttributeIdentifier.values(), FieldDataType.RELATIONSHIP, 0),
		ACQUISITION_LOT_RECORD_RELATIONSHIP ("acquisition_lot_record_relationship", AcquisitionLotRecordRelationshipAttributeIdentifier.values(), FieldDataType.RELATIONSHIP, 0),
		OBJECT_OBJECT_RELATIONSHIP ("object_object_relationship", ObjectObjectRelationshipAttributeIdentifier.values(), FieldDataType.RELATIONSHIP, 0),
		OBJECT_OBJECT_RELATIONSHIP_NOTES ("object_object_relationship_notes", ObjectObjectRelationshipNotesAttributeIdentifier.values(), FieldDataType.TEXT, 0),
		PLACE_MARK_NAME ("place_mark_name", PlaceMarkNameAttributeIdentifier.values(), FieldDataType.VARCHAR, 200),
		LATITUDE ("latitude", LatitudeAttributeIdentifier.values(), FieldDataType.NUMBER, 9),
		LONGITUDE ("longitude", LongitudeAttributeIdentifier.values(), FieldDataType.NUMBER, 10),
		DATUM ("datum", DatumAttributeIdentifier.values(), FieldDataType.GLOBAL_TERM, 200),
		ZOOM_LEVEL ("zoom_level", ZoomLevelAttributeIdentifier.values(), FieldDataType.NUMBER, 2),
		PLACE_MARK_NOTES ("place_mark_notes", PlaceMarkNotesAttributeIdentifier.values(), FieldDataType.TEXT, 65535),
		PLACE_MARK_ACCOUNT_RELATIONSHIP ("place_mark_account_relationship", PlaceMarkAccountRelationshipAttributeIdentifier.values(), FieldDataType.ACCOUNT_RELATIONSHIP, 0);
		
		public enum FieldDataType {
			TERM,
			DATE,
			TEXT,
			VARCHAR,
			NUMBER,
			GLOBAL_TERM,
			POINTER,
			RELATIONSHIP,
			ACCOUNT_RELATIONSHIP;
		}
		
		enum TransferredToAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private TransferredToAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AccessionDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private AccessionDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SubjectAssociationDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SubjectAssociationDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AssociationKeywordAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private AssociationKeywordAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum BriefDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private BriefDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CarrierTypeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CarrierTypeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CataloguedDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CataloguedDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CataloguerAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CataloguerAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ClassificationAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ClassificationAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CommentsAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CommentsAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CommentsDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CommentsDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CommentsPersonAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CommentsPersonAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CompletedTreatmentDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CompletedTreatmentDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CompletedTreatmentDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CompletedTreatmentDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CompletedTreatmentKeywordAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CompletedTreatmentKeywordAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CompletedTreatmentNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CompletedTreatmentNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CompletedTreatmentPartReferenceAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CompletedTreatmentPartReferenceAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CompletedTreatmentPersonAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CompletedTreatmentPersonAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ConditionAtAcquisitionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ConditionAtAcquisitionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ConditionDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ConditionDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ConditionKeywordAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ConditionKeywordAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ConditionNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ConditionNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ConditionPartReferenceAspectAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ConditionPartReferenceAspectAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ConditionPersonAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ConditionPersonAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ConservationPriorityAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ConservationPriorityAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ControllingInstitutionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ControllingInstitutionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CopyrightExpiryDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CopyrightExpiryDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CopyrightNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CopyrightNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CopyrightOwnerAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CopyrightOwnerAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CopyrightStartDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CopyrightStartDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CopyrightTypeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CopyrightTypeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CreditLineAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CreditLineAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CurrentConditionDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CurrentConditionDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum CurrentRecordStatusAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private CurrentRecordStatusAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum DateMadeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private DateMadeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum DeaccessionAuthoriserAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private DeaccessionAuthoriserAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum DeaccessionAuthoriserPositionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private DeaccessionAuthoriserPositionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum DeaccessionDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private DeaccessionDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum DeaccessionMethodAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private DeaccessionMethodAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum DeaccessionProceedsAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private DeaccessionProceedsAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum DeaccessionReasonAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private DeaccessionReasonAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum DeaccessionStatusAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private DeaccessionStatusAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum DepartmentAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private DepartmentAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum DisplayDetailsAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private DisplayDetailsAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum DublinCoreAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private DublinCoreAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum EditionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private EditionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum FieldCollectionDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private FieldCollectionDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum FieldCollectionTimeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private FieldCollectionTimeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum FieldCollectionMethodAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private FieldCollectionMethodAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum FieldCollectionDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private FieldCollectionDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum FieldCollectionPlaceKeywordAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private FieldCollectionPlaceKeywordAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum FieldCollectionReferenceAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private FieldCollectionReferenceAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum FieldCollectorAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private FieldCollectorAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum FieldCollectorRoleAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private FieldCollectorRoleAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum FilmTypeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private FilmTypeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum FormatAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private FormatAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum FunderAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private FunderAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum FunderAmountAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private FunderAmountAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum FunderTypeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private FunderTypeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum GeneralFlagAdminAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private GeneralFlagAdminAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum GeneralNotesAcquisitionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private GeneralNotesAcquisitionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum GeneralNotesAdminAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private GeneralNotesAdminAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum GeneralNotesObjectAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private GeneralNotesObjectAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum GeneralNotesPlaceMarkAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private GeneralNotesPlaceMarkAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum GeologicalAgeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private GeologicalAgeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum GeologicalFormationAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private GeologicalFormationAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum HabitatDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private HabitatDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum HabitatKeywordAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private HabitatKeywordAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum HandlingDetailsAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private HandlingDetailsAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum InscriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private InscriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum IsbnIssnAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private IsbnIssnAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ItemCountAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ItemCountAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ItemCountNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ItemCountNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LanguageAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LanguageAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LocationAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LocationAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LocationNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LocationNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum MarineEnvironmentAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private MarineEnvironmentAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum MeasurementAspectAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private MeasurementAspectAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum MeasurementDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private MeasurementDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum MeasurementPartAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private MeasurementPartAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum MeasurementUnitAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private MeasurementUnitAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum MeasurementValueAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private MeasurementValueAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum MediumDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private MediumDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum MediumMaterialAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private MediumMaterialAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum MountingFramingAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private MountingFramingAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum NamedCollectionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private NamedCollectionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ObjectNumberAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ObjectNumberAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ObjectTypeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ObjectTypeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum OtherMakerContributorAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private OtherMakerContributorAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum OtherMakerContributorRoleAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private OtherMakerContributorRoleAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum OtherNameTitleAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private OtherNameTitleAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum OtherNameTitleNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private OtherNameTitleNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum OtherNameTitleTypeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private OtherNameTitleTypeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum OtherNumberAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private OtherNumberAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum OtherNumberTypeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private OtherNumberTypeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PackingDetailsAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PackingDetailsAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PartIdAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PartIdAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PartDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PartDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PartHoldingsAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PartHoldingsAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PhysicalCharacteristicsAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PhysicalCharacteristicsAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PlaceMarkAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PlaceMarkAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PlaceMadeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PlaceMadeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PlaceNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PlaceNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PortfolioTitleAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PortfolioTitleAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PriceForeignAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PriceForeignAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PriceLocalAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PriceLocalAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PrimaryCreatorMakerAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PrimaryCreatorMakerAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PrimaryCreatorMakerRoleAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PrimaryCreatorMakerRoleAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ProposedTreatmentDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ProposedTreatmentDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ProposedTreatmentDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ProposedTreatmentDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ProposedTreatmentKeywordAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ProposedTreatmentKeywordAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ProposedTreatmentNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ProposedTreatmentNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ProposedTreatmentPartReferenceAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ProposedTreatmentPartReferenceAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ProposedTreatmentPersonAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ProposedTreatmentPersonAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ProvenanceDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ProvenanceDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ProvenanceDetailsAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ProvenanceDetailsAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ProvenancePersonAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ProvenancePersonAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ProvenancePlaceAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ProvenancePlaceAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PublisherAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PublisherAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PublicationDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PublicationDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PublicationPlaceAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PublicationPlaceAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RecordStatusAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private RecordStatusAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RelatedAcquisitionNumberAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private RelatedAcquisitionNumberAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RelatedAcquisitionNumberTypeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private RelatedAcquisitionNumberTypeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RelatedObjectAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private RelatedObjectAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RelatedObjectNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private RelatedObjectNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RelatedObjectTypeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private RelatedObjectTypeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RestrictionTypeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private RestrictionTypeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RiskAssessedByAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private RiskAssessedByAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RiskAssessmentAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private RiskAssessmentAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RiskAssessmentDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private RiskAssessmentDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RiskAssessmentNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private RiskAssessmentNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RiskFactorAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private RiskFactorAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SeriesNumberAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SeriesNumberAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SeriesTitleAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SeriesTitleAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SexAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SexAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ShoreEnvironmentAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ShoreEnvironmentAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SignificanceAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SignificanceAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SiteDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SiteDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SiteKeywordAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SiteKeywordAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SoundAudioAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SoundAudioAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SpecimenAgeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SpecimenAgeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SpecimenCategoryAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SpecimenCategoryAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SpecimenCategoryNotesCountAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SpecimenCategoryNotesCountAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SpecimenDetailsAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SpecimenDetailsAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SpineLabelAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SpineLabelAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum StorageDetailsAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private StorageDetailsAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum StoragePreservationMethodAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private StoragePreservationMethodAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum StratigraphyDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private StratigraphyDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum StratigraphyKeywordAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private StratigraphyKeywordAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum StyleIconographyAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private StyleIconographyAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SubjectKeywordAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SubjectKeywordAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum TaxonDeterminedByAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private TaxonDeterminedByAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum TaxonDeterminedDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private TaxonDeterminedDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum TaxonomicClassificationAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private TaxonomicClassificationAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum TaxonomicClassificationNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private TaxonomicClassificationNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum TaxonomicQualifierAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private TaxonomicQualifierAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum TaxonomicTypeIndicatorAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private TaxonomicTypeIndicatorAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum TechniqueAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private TechniqueAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum NameAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private NameAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum WebPublicDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private WebPublicDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ExternalDatabaseIdAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ExternalDatabaseIdAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SimpleSummaryAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SimpleSummaryAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SubjectAssociationEventAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SubjectAssociationEventAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum TagAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private TagAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum FamilyNameAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private FamilyNameAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum GivenNameAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private GivenNameAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AdditionalNamesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private AdditionalNamesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum KnownAsAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private KnownAsAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum TitleAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private TitleAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PostnominalsAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PostnominalsAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum OrganisationNameAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private OrganisationNameAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum SortNameAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private SortNameAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum OtherNameTypeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private OtherNameTypeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum OtherNameAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private OtherNameAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum OtherNameDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private OtherNameDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum DateOfBirthAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private DateOfBirthAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum DateOfDeathAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private DateOfDeathAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PlaceOfBirthAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PlaceOfBirthAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PlaceOfDeathAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PlaceOfDeathAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum UrlAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private UrlAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RelatedRecordAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private RelatedRecordAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RelationshipAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private RelationshipAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RelatedNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private RelatedNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum JobTitleAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private JobTitleAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum JobRoleAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private JobRoleAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum GeneralNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private GeneralNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ActiveDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ActiveDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ActivePlaceAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ActivePlaceAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ActiveNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ActiveNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum NationalityAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private NationalityAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum NationalityDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private NationalityDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum NationalityNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private NationalityNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum EthnicityAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private EthnicityAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AcqNumberAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private AcqNumberAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AcqMethodAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private AcqMethodAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AcqReasonAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private AcqReasonAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AcqDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private AcqDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AcqAuthorityAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private AcqAuthorityAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AcqSourceAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private AcqSourceAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AcqSourceRoleAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private AcqSourceRoleAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LotValuationAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LotValuationAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LotPriceLocalAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LotPriceLocalAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LotPriceForeignAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LotPriceForeignAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LotDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LotDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AcqValuationAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private AcqValuationAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LegalStatusAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LegalStatusAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LoanInAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LoanInAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LoanOutAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LoanOutAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ExhibitionHistoryAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ExhibitionHistoryAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ResearchAttributionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ResearchAttributionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ResearchAttributionDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ResearchAttributionDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ResearchAttributionPersonAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ResearchAttributionPersonAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum FieldCollectionNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private FieldCollectionNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum GeologicalFormationDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private GeologicalFormationDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum GeologicalAgeDescriptionAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private GeologicalAgeDescriptionAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum GeologicalFormAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private GeologicalFormAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PeriodAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PeriodAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum TypeIdReliabilityLevelAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private TypeIdReliabilityLevelAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AcqStatusAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private AcqStatusAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AcqFunderAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private AcqFunderAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AcqFundingAmountAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private AcqFundingAmountAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AcqFundingTypeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private AcqFundingTypeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ProductionNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ProductionNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ValuationTypeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ValuationTypeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ValuationMethodAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ValuationMethodAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ValuationAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ValuationAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ValuationValuerAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ValuationValuerAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ValuationDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ValuationDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ValuationForeignCurrencyTypeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ValuationForeignCurrencyTypeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ValuationForeignCurrencyValueAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ValuationForeignCurrencyValueAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ValuationExchangeRateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ValuationExchangeRateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ValuationNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ValuationNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LatestValuationAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LatestValuationAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LatestAppraisalValuationAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LatestAppraisalValuationAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LatestConservationValuationAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LatestConservationValuationAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LatestInsuranceValuationAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LatestInsuranceValuationAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LatestOutgoingLoanValuationAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LatestOutgoingLoanValuationAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LocationHistoryAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LocationHistoryAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LocationHistoryDateAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LocationHistoryDateAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LocationHistoryNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LocationHistoryNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum RecordAcquisitionLotRelationshipAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE),
			RELATED_RECORD_ID (AttributeIdentifier.RELATED_RECORD_ID),
			RELATED_OBJECT_RECORD (AttributeIdentifier.RELATED_OBJECT_RECORD);			
			
			private final AttributeIdentifier attributeIdentifier;
			
			private RecordAcquisitionLotRelationshipAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum AcquisitionLotRecordRelationshipAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE),
			RELATED_RECORD_ID (AttributeIdentifier.RELATED_RECORD_ID),
			RELATED_OBJECT_RECORD (AttributeIdentifier.RELATED_OBJECT_RECORD);			
			
			private final AttributeIdentifier attributeIdentifier;
			
			private AcquisitionLotRecordRelationshipAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ObjectObjectRelationshipAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE),
			RELATED_RECORD_ID (AttributeIdentifier.RELATED_RECORD_ID),
			RELATED_OBJECT_RECORD (AttributeIdentifier.RELATED_OBJECT_RECORD);			
			
			private final AttributeIdentifier attributeIdentifier;
			
			private ObjectObjectRelationshipAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ObjectObjectRelationshipNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ObjectObjectRelationshipNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PlaceMarkNameAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PlaceMarkNameAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LatitudeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LatitudeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum LongitudeAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private LongitudeAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum DatumAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private DatumAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum ZoomLevelAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private ZoomLevelAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PlaceMarkNotesAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE);

			private final AttributeIdentifier attributeIdentifier;
			
			private PlaceMarkNotesAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		enum PlaceMarkAccountRelationshipAttributeIdentifier implements FieldAttributeMapper {
			LABEL (AttributeIdentifier.LABEL),
			VALUE (AttributeIdentifier.VALUE),
			RELATED_ACCOUNT_ID (AttributeIdentifier.RELATED_ACCOUNT_ID);			
			
			private final AttributeIdentifier attributeIdentifier;
			
			private PlaceMarkAccountRelationshipAttributeIdentifier(AttributeIdentifier attributeIdentifier) {
				this.attributeIdentifier = attributeIdentifier;
			}
			
			public AttributeIdentifier getAttributeIdentifier() {
				return this.attributeIdentifier;
			}
			
			public static Boolean isValid(String identifier) {
				try { 
					AttributeIdentifier.valueOf(identifier.toUpperCase()); 
				} catch (IllegalArgumentException e) { 
					return false;
				}
				return true;
			}

			public String toString() {
				return this.attributeIdentifier.toString();
			}
		}

		private final String identifier;
		private final Enum<?>[] validAttributeIdentifiers;
		private final FieldDataType fieldDataType;
		private final Integer fieldLength;
		
		private FieldIdentifier(String identifier, Enum<?>[] validAttributeIdentifers, FieldDataType fieldDataType, Integer fieldLength) {
			this.identifier = identifier;
			this.validAttributeIdentifiers = validAttributeIdentifers;
			this.fieldDataType = fieldDataType;
			this.fieldLength = fieldLength;
		}
		
		public String getFieldIdentifier() {
			return this.identifier;
		}
		
		public Enum<?>[] getValidAttributeIdentifiers() {
			return this.validAttributeIdentifiers;
		}
		
		public FieldDataType getFieldDataType() {
			return this.fieldDataType;
		}
		
		public Integer getFieldLength() {
			return this.fieldLength;
		}
		
		public static Boolean isValid(String identifier) {
			try { 
				getFieldIdentifier(identifier); 
			} catch (IllegalArgumentException e) { 
				return false;
			}
			return true;
		}
		
		public static FieldIdentifier getFieldIdentifier(String identifier) {
			return valueOf(identifier.toUpperCase());
		}

		public String toString() {
			return this.identifier;
		}
	}
	
}